datablock StaticShapeData(EmptyShapeData) {
	shapeFile = "base/data/shapes/empty.dts";
};

function NodeSO(%position, %group) {
	if (!isObject(%group)) {
		if (%group $= "") {
			if (!isObject(%group = NodeGroup)) {
				%group = new scriptGroup(NodeGroup) {
					class = NodeSG;
				};

				MissionCleanup.add(%group);
			}
		}
		else {
			error( "ERROR: Invalid node group." );
			return;
		}
	}

	%obj = new scriptObject() {
		class = NodeSO;

		position = %position;
		neighbors = 0;
	};

	%group.add(%obj);

	if (%group.debug) {
		%obj.createDebugObjects();
	}

	return %obj;
}

function NodeSO::onAdd(%this) {
	%this.shape = new StaticShape() {
		position = %this.position;
		datablock = EmptyShapeData;

		node = %this;
	};
}

function NodeSO::onRemove(%this) {
	%this.shape.delete();
	%this.destroyDebugObjects();

	for ( %i = %this.neighbors - 1 ; %i >= 0 ; %i-- )
	{
		if ( !isObject( %this.neighbor[ %i ] ) )
		{
			continue;
		}

		%this.removeNeighbor( %this.neighbor[ %i ] );
	}
}

function NodeSO::addNeighbor(%this, %node) {
	if ( %node.class !$= "NodeSO" )
	{
		error( "ERROR: Invalid node." );
		return;
	}

	if ( ( %group = %this.getGroup() ) != %node.getGroup() )
	{
		error( "ERROR: Node belongs to another group." );
		return;
	}

	%node = %node.getID();

	if ( %this == %node )
	{
		error( "ERROR: Cannot connect a node to itself." );
		return;
	}

	if ( !%this.isNeighbor[ %node ] )
	{
		%this.neighbor[ %this.neighbors ] = %node;
		%this.neighbors++;

		%this.isNeighbor[ %node ] = true;

		if ( %group.debug )
		{
			%this.createDebugObjects();
		}
	}

	if ( !%node.isNeighbor[ %this ] )
	{
		%node.addNeighbor( %this );
	}
}

function NodeSO::removeNeighbor( %this, %node )
{
	if ( %node.class !$= "NodeSO" )
	{
		error( "ERROR: Invalid node." );
		return;
	}

	%node = %node.getID();

	if ( %this.isNeighbor[ %node ] )
	{
		for ( %i = 0 ; %i < %this.neighbors ; %i++ )
		{
			if ( %found )
			{
				%this.neighbor[ %i ] = %this.neighbors[ %i + 1 ];
			}
			else if ( %this.neighbor[ %i ] == %node )
			{
				%found = true;
				%i--;

				continue;
			}
		}

		if ( %found )
		{
			%this.neighbors--;
			%this.neighbor[ %this.neighbors ] = "";
		}

		%this.isNeighbor[ %node ] = "";

		if ( isObject( %this.debugLine[ %node ] ) )
		{
			%this.debugLine[ %node ].delete();
			%this.debugLine[ %node ] = "";
		}
	}

	if ( %node.isNeighbor[ %this ] )
	{
		%node.removeNeighbor( %this );
	}
}

function NodeSO::createDebugObjects( %this )
{
	%color = "0 0.75 0";

	if ( !isObject( %this.debugCube ) )
	{
		// %this.debugCube = createCube( %this.position, "0.5 0.5 0.4", %color SPC 1 );
		// %this.debugCube = createCube( %this.position, "0.5 0.5 0.4", "0 0 0 0" );
	}

	%line = vectorScale( %color, 0.5 );
	%line = vectorAdd( %line, "0.5 0.5 0.5" );

	for ( %i = 0 ; %i < %this.neighbors ; %i++ )
	{
		%neighbor = %this.neighbor[ %i ];

		if ( !isObject( %neighbor ) )
		{
			continue;
		}

		if ( !isObject( %this.debugLine[ %neighbor ] ) )
		{
			if ( isObject( %neighbor.debugLine[ %this ] ) )
			{
				%this.debugLine[ %neighbor ] = %neighbor.debugLine[ %this ];
				continue;
			}

			%a = %this.position;
			%b = %neighbor.position;

			// %this.debugLine[ %neighbor ] = createLine( %a, %b, 0.005, %line SPC 1 );
			%this.debugLine[ %neighbor ] = createLine( %a, %b, 0.005, "0 0 0 0" );
			%neighbor.debugLine[ %this ] = %this.debugLine[ %neighbor ];
		}
	}
}

function NodeSO::destroyDebugObjects( %this )
{
	if ( isObject( %this.debugCube ) )
	{
		%this.debugCube.delete();
		%this.debugCube = "";
	}

	for ( %i = 0 ; %i < %this.neighbors ; %i++ )
	{
		%neighbor = %this.neighbor[ %i ];

		if ( isObject( %this.debugLine[ %neighbor ] ) )
		{
			%this.debugLine[ %neighbor ].delete();

			%this.debugLine[ %neighbor ] = "";
			%neighbor.debugLine[ %this ] = "";
		}
	}
}

function NodeSG::setDebug( %this, %bool )
{
	%this.debug = %bool ? true : false;
	%count = %this.getCount();

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%node = %this.getObject( %i );

		if ( %bool )
		{
			%node.createDebugObjects();
		}
		else
		{
			%node.destroyDebugObjects();
		}
	}
}

function NodeSG::findNearest(%this, %position, %range, %visible) {
	initContainerRadiusSearch(%position, %range, $TypeMasks::StaticShapeObjectType);

	while (isObject(%obj = containerSearchNext())) {
		if (isObject(%obj.node)) {
			if (%visible) {
				%ray = containerRayCast(
					vectorAdd(%position, "0 0 1"),
					vectorAdd(%obj.position, "0 0 1"),
					$TypeMasks:FxBrickObjectType
				);

				if (%ray !$= 0) {
					continue;
				}
			}

			return %obj.node;
		}
	}

	return -1;
}

function loadNodes( %file, %debug )
{
	%mode = 0;
	%nodes = 0;

	if ( !isFile( %file ) )
	{
		error( "ERROR: Cannot find file." );
		return;
	}

	%fo = new fileObject();
	%fo.openForRead( %file );

	if ( isObject( nodeGroup ) )
	{
		nodeGroup.delete();
	}

	%group = new scriptGroup( nodeGroup )
	{
		class = "NodeSG";
		debug = false;
	};

	missionCleanup.add( %group );

	while ( !%fo.isEOF() )
	{
		%line = %fo.readLine();

		if ( %line $= "neighbors" )
		{
			%mode = 1;
			continue;
		}

		if ( %mode == 0 )
		{
			%node[ %nodes ] = NodeSO( %line );
			%nodes++;
		}
		else if ( %mode == 1 )
		{
			%node[ getWord( %line, 0 ) ].addNeighbor( %node[ getWord( %line, 1 ) ] );
		}
	}

	%fo.close();
	%fo.delete();

	if ( %debug )
	{
		NodeGroup.setDebug( %debug );
	}
}

function saveNodes( %file )
{
	if ( !isWriteableFileName( %file ) )
	{
		error( "ERROR: Cannot write to file." );
		return;
	}

	%fo = new fileObject();
	%fo.openForWrite( %file );

	%count = nodeGroup.getCount();

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%node = nodeGroup.getObject( %i );
		%id[ %node ] = %i;

		%fo.writeLine( %node.position );
	}

	%fo.writeLine( "neighbors" );

	for ( %i = 0 ; %i < %count ; %i++ )
	{
		%node = nodeGroup.getObject( %i );

		for ( %j = 0 ; %j < %node.neighbors ; %j++ )
		{
			%neighbor = %node.neighbor[ %j ];

			if ( !isObject( %neighbor ) || %rel[ %node, %neighbor ] || %rel[ %neighbor, %node ] )
			{
				continue;
			}

			%rel[ %node, %neighbor ] = true;
			%fo.writeLine( %id[ %node ] SPC %id[ %neighbor ] );
		}
	}

	%fo.close();
	%fo.delete();
}

function Player::getNearestNode(%this) {
	if ($Sim::Time - %this.lastNode > 0.5) {
		%this.node = NodeGroup.findNearest(%this.position, 16, 1);

		if (!isObject(%this.node)) {
			%this.node = NodeGroup.findNearest(%this.position, 16);
		}

		%this.lastNode = $Sim::Time;
	}

	return %this.node;
}

function Projectile::getNearestNode(%this) {
	if ($Sim::Time - %this.lastNode > 0.5) {
		%this.node = NodeGroup.findNearest(%this.position, 16, 1);

		if (!isObject(%this.node)) {
			%this.node = NodeGroup.findNearest(%this.position, 16);
		}

		%this.lastNode = $Sim::Time;
	}

	return %this.node;
}